/**************************************************************************
lecroy_example.c
Version 1.00
Copyright (C) 2003  Steve D. Sharples

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The author's email address is steve.sharples@nottingham.ac.uk

***************************************************************************

Example program that demonstrates communication via TCP/IP with an ethernet-
enabled LeCroy DSO. Based on the main() function of "net_con.cpp" written
by LeCroy.

See "lecroy_tcp.c" and "lecroy_tcp.h" for additional comments and usage

**************************************************************************/


#include "lecroy_tcp.h"
#include <stdlib.h> 
#include <unistd.h>

#include <stdio.h>
#include <string.h>
using namespace std;

#define	MAX_ST	512000	/* maximum message string. no picked from thin air */
int getnbytes(char *inbuf){
  char mstr[10];
  strncpy(mstr,&inbuf[2],8);
  mstr[8]=0;
  return atoi(mstr);
}
  
int LECROY_TCP_ScreenDump(int sockfd, char* fname){	
  char inbuf[MAX_ST]; 
  LECROY_TCP_write(sockfd, "TSCRN? PNG\n");
  LECROY_TCP_read(sockfd, inbuf, sizeof(inbuf), MAX_TCP_READ);
  FILE *fp=fopen(fname,"w");
  int bcount= getnbytes(inbuf); 
  printf("Writing %d bytes to file!\n",bcount); 
  fwrite(&inbuf[10],bcount,1,fp);
  fclose(fp);
  return bcount;
}

int main ()
{
	char ip_address[50];
	int sockfd;
	char outbuf[MAX_ST];
	char inbuf[MAX_ST];
	float fvalue;
	int ivalue;


	/* OUR_SCOPE_IP is #define'd in lecroy_tcp.h, so is MAX_TCP_CONNECT */
	sprintf(ip_address, OUR_SCOPE_IP);

	printf("\nlecroy_example: about to connect to DSO...\n");

	/* how to "set up the scope for ethernet communication" */
	sockfd=LECROY_TCP_connect(ip_address, MAX_TCP_CONNECT);
	if (sockfd<0) {
		printf("\nCould not connect to the scope on IP: %s\n",ip_address);
		return 1;
		}

	printf("....connected.\n");
         
	if (argc<2){
          printf("Usage screendump: %s <file.png> ");
          printf("Usage waveform acquire: %s <neve> <file.dat> ");
	  return 0;
	}

	int neve =atoi(argv[1]);
	if (neve==0){
	  LECROY_TCP_ScreenDump(sockfd,argv[1]);
	  return 0;
	}

	LECROY_TCP_write(sockfd, "MLEN 1K\n");	  
	LECROY_TCP_write(sockfd, "DTPOINTS 200\n");
	LECROY_TCP_write(sockfd, "DTSTART 50\n");
	LECROY_TCP_write(sockfd, "WAVESRC CH1\n");
	LECROY_TCP_write(sockfd, "DTFORM ASCII\n");
	//LECROY_TCP_write(sockfd, "*TRG\n");
	LECROY_TCP_write(sockfd, "TRMD NORMAL\n");
	const char cmd[][11]={"*idn?","DTFORM?","DTSTART?","DTBORD?","DATE?","DTWAVE?",
			      "dtpoints?","mlen?","ACQ?","WAVESRC?","DTINF?","TRMD?"};
	for (int i=0;i<12;i++){
	  /* example of writing to the scope) */
	  sprintf(outbuf,"%s\n",cmd[i]);
	  LECROY_TCP_write(sockfd, outbuf);
	  /* example of getting the raw data back from the scope */
	  LECROY_TCP_read(sockfd, inbuf, sizeof(inbuf), MAX_TCP_READ);
	  printf ("%s=>%s\n", outbuf,inbuf);

	}


	// determine which channels are switched on
	LECROY_TCP_write(sockfd, "DTINF?\n");
	LECROY_TCP_read(sockfd, inbuf, sizeof(inbuf), MAX_TCP_READ);
        char *tok=NULL;
	int ison[4]={0,0,0,0};
	int i=0;
	for (tok=strtok(inbuf, ","); tok; tok=strtok(NULL,",")) {
          int ch=0;
          switch (i){
	  case 6:  ch=0; break;
	  case 10: ch=1; break;
	  case 14: ch=2; break;
	  case 18: ch=3; break;
	  default: ch=-1;break;
	  }
	  i++;
	  if (ch<0) continue;
          if (strstr(tok,"Available")!=NULL ) ison[ch]=1; else ison[ch]=0;
	}



    

	LECROY_TCP_write(sockfd, "DTFORM BYTE\n");

        FILE *fp=fopen([argv[1],"wb");
	for (int i=0;i<neve;i++){
          int hlen=sizeof(int)*4;
          int ehdr[4]={0x1,hlen,i,0};        
          fwrite(ehdr,hlen,1,fp);
	  LECROY_TCP_write(sockfd, "TRMD SINGLE\n");
	  usleep(100000);
	
	  //LECROY_TCP_write(sockfd, "TRMD NORMAL\n");
	  //LECROY_TCP_write(sockfd, "TRMD STOP\n");
	  for ( int ch=0;ch<4;ch++){
            if (!ison[ch]) continue;
            sprintf(outbuf,"WAVESRC CH%d\n",ch+1);
	    LECROY_TCP_write(sockfd, outbuf);
	    LECROY_TCP_write(sockfd, "DTWAVE?\n");
	    LECROY_TCP_read(sockfd, inbuf, sizeof(inbuf), MAX_TCP_READ);

	    int hdr[4]={0x2,getnbytes(inbuf)+hlen,ch,0};        
	    fwrite(hdr,hlen,1,fp);
	    fwrite(inbuf,hdr[1]-hlen,1,fp);
	    if (i%1000!=0){
	      inbuf[10]=0;
	      printf ("[%d] ch=%d %s\n", i,ch,inbuf);
	    } else {
	      printf("WAVEFORM Data %d\n",ch);
	      char *data=(char *) (&inbuf[10]);
	      for (int k=0;k<getnbytes(inbuf);k++){
		printf("%d\t",data[k]);
	      }
	      printf("\n");
	    }
	  }
        }
	fclose(fp);

	LECROY_TCP_disconnect(sockfd);
	printf("Successfully disconnected... bye!\n");
	return 0;
}
